#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Module stub that keeps the one centraliced variable that gives
the current version number.

It is read by multiple build scripts, too.
"""

_current_year = 2025

__title__ = 'ecpet'
__copyright__ = f"(c) 2017-{_current_year} Clemens Drüe, Univerität Trier"
__release__ = '2.1.0-beta1'

